#!/bin/bash

# Copyright (c) 2016-2023 DEEPHAVEN DATA LABS - PATENT PENDING

# This script is automatically generated by iris-plugin-tools.
# Your edits to it will be erased upon upgrade / uninstallation of this plugin.
# If you want to make persistent changes, please contact the developer
# of this plugin, the developer of the plugin tools, or your Deephaven representative.
# *Your changes will be overwritten BEFORE any upgrade operation is performed*.

set -o errexit
set -o pipefail
umask 002


# disable trace logging until after we've redirected stdout and stderr
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
SCRIPT_NAME=$(basename "${BASH_SOURCE[0]}")
INSTALL_TIME=${INSTALL_TIME:-$(date "+%Y%m%d%H%M%S")}
export INSTALL_TIME

current_user_id=$(id -u)
current_group_id=$(id -g)
irisadmin_user_id=$(id -u irisadmin)
root_user_id=$(id -u root)
irisadmin_group_id=$(getent group irisadmin | awk -F: '{ print $3 }')
desired_user_id=$(id -u irisadmin)
desired_group_id=$(getent group dbmergegrp | awk -F: '{ print $3 }')

if (( current_user_id == irisadmin_user_id)); then
    as_irisadmin=" "
else
    as_irisadmin="sudo -u irisadmin"
fi

if (( current_user_id == desired_user_id )); then
    if (( current_group_id == desired_group_id )); then
        as_desired_user=" "
    else
        echo "$(id -un):$(id -ug) Cannot run ${SCRIPT_NAME} as irisadmin:dbmergegrp"
        exit 77
    fi
else
    as_desired_user="sudo -u irisadmin  "
fi

if (( current_user_id == root_user_id )); then
    as_admin_user=" "
else
    as_admin_user="sudo "
fi

LOG_DIR="/var/log/deephaven/plugins/example-plugin"
LOG_DIR="${LOG_DIR}/installation"
EXECUTION_DIR="${LOG_DIR}/executions"
LOG_FILE="${LOG_DIR}/${SCRIPT_NAME}-${INSTALL_TIME}.log"
EXECUTION_FILE="${EXECUTION_DIR}/${SCRIPT_NAME}.${INSTALL_TIME}"

# Create LOG_ROOT as irisadmin, then create individual logging directories as the desired user.
$as_irisadmin mkdir -p "${LOG_DIR}"
$as_irisadmin chmod 777 "${LOG_DIR}"
$as_desired_user mkdir -p "${LOG_DIR}"
$as_desired_user mkdir -p "${EXECUTION_DIR}"
$as_irisadmin chmod 755 "${LOG_DIR}"

$as_desired_user touch "${EXECUTION_FILE}"
$as_desired_user touch "${LOG_FILE}"
$as_desired_user chmod 777 "${LOG_FILE}"
$as_desired_user ln -sf "${EXECUTION_FILE}" ${EXECUTION_DIR}/current.${SCRIPT_NAME}.log
$as_desired_user ln -sf "${LOG_FILE}" ${LOG_DIR}/current.${SCRIPT_NAME}.log

$as_desired_user chmod -R 755 "${LOG_DIR}"
$as_desired_user chmod -R 755 "${EXECUTION_DIR}"

### This "exec > >(tee..." will send all script to log file as well as screen, preserving original stdout/err as &3 n &4
exec 3>&1 4>&2 \
     1> >( $as_desired_user tee -a "${LOG_FILE}" >&3  )  \
     2> >( grep --line-buffered -v "+ set +o xtrace" | grep --line-buffered -v "+ log_" | $as_desired_user tee -a "${LOG_FILE}" | grep -E -v "^\+\+* " >&4  )
# The first grep removes + set +o xtrace, the second removes log_* calls; then sends to log file, then erase lines beginning w/ +, then send to "real stderr"



START_FILE="${EXECUTION_DIR}/SCHEMA-start"
SUCCESS_FILE="${EXECUTION_DIR}/SCHEMA-success"
FAIL_FILE="${EXECUTION_DIR}/SCHEMA-fail"


set -o xtrace

function start() {

    if $as_desired_user test -r "${START_FILE}"; then
        if [[ "$($as_desired_user cat ${START_FILE})" == "10@${INSTALL_TIME}" ]]; then
            log_debug "activate_schema.sh already run for activation 10@${INSTALL_TIME}"
            exit 0
        fi
    fi

    log_info "Starting ${SCRIPT_NAME}"
    
    ### Save command
    echo "${SCRIPT_DIR}/${SCRIPT_NAME} ${@}" | $as_desired_user tee "${EXECUTION_FILE}" > /dev/null
    log_debug ""
    log_debug "Script Invoked:"
    log_debug "${SCRIPT_DIR}/${SCRIPT_NAME} ${@}"
    log_debug 

    $as_desired_user rm -f "${FAIL_FILE}"
    $as_desired_user rm -f "${SUCCESS_FILE}"
    echo -n "10@${INSTALL_TIME}" | $as_desired_user tee "${START_FILE}" > /dev/null

}

function finish() {
    local target="${1:-0}"
    if [ "0" == "$target" ]; then
        target="${SUCCESS_FILE}"
    else
        target="${FAIL_FILE}"
    fi
    log_info "Finishing ${SCRIPT_NAME}"
    echo -n "10@${INSTALL_TIME}" | $as_desired_user tee "$target" > /dev/null
}

function log_debug() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S") | --- $@" | $as_desired_user tee -a ${LOG_FILE} > /dev/null
    set -o xtrace
}

function log_info() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S")  | --- $@" >&1
    set -o xtrace
}

function log_error() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S")  | --- $@" >&2
    set -o xtrace
}

start


/etc/sysconfig/deephaven/plugins/example-plugin/bin/activate_basic.sh 1>&3 2>&4


# Require that a given file or directory is owned by expected user
function require_owner() {
  local func_name='require_owner'
  # The file whose ownership we need to create
  local file="${1}"
  # The ownership to apply to $file
  local ownership
  ownership="${2:-irisadmin:irisadmin}"
  # The expected owner user
  local expected_user
  expected_user="${ownership%%:*}"
  # The expected owner group
  local expected_group
  expected_group="${ownership##*:}"
  # Any additional args to pass to chgrp / chown
  local extra_args
  shift 2
  extra_args="${*}"
  
  file="$(readlink -m "${file}")"
  if test -e "${file}" && ! check_file_owner "${file}" ${expected_user} ${expected_group}; then
      if check_file_owner "${file}" ${expected_user}; then
          sudo -n -u ${expected_user} -g ${expected_group} chgrp ${extra_args} ${expected_group} "${file}"
      else
          $as_admin_user chown ${extra_args} ${ownership} "${file}"
      fi
  fi

} # end function require_owner

# Returns the owner of a file, and if 2nd arg provided, returns 0 if current owner == expected owner
function check_file_owner() {
  local func_name='check_file_owner'
  # The file to check for ownership
  local file
  file="$(readlink -m "${1}")"
  # [Optional] an expected owner user to check against
  local expected_user="${2}"
  # [Optional] an expected owner group to check against
  local expected_group="${3}"
  # Any necessary sudo -u needed to be able to read the file
  local as_user
  as_user="sudo -n -u ${expected_user:-irisadmin}"
  # The actual owner user of the file, if it exists
  local real_user
  # The actual owner group of the file, if it exists
  local real_group
  
  file="$(readlink -m "${file}")"
  if (( $(id -u) == $(id -u ${expected_user}) )); then as_user=''; fi
  if ${as_user} test -e ${file}; then
      if ${as_user} test -n ${expected_user}; then
          real_user="$(${as_user} stat -c %U "${file}" || sudo -n stat -c %U "${file}" )"
          if [[ "${expected_user}" == "${real_user}" ]]; then
              if [[ -n "${expected_group}" ]]; then
                  real_group="$(${as_user} stat -c %G "${file}" || stat -c %G "${file}" )"
                  [[ "${expected_group}" == "${real_group}" ]] && return 0 || return 1
              fi
              return 0
          fi
          return 1
      fi
      return 0
  fi
  return 1
} # end function check_file_owner


iris="$as_irisadmin /usr/illumon/latest/bin/iris"
require_owner /etc/sysconfig/deephaven/plugins/example-plugin/schema irisadmin:dbmergegrp -R
# Any version on or after 1.20190816 must import our schemas using an alternate syntax
dh_version="$(cat /usr/illumon/latest/etc/IRIS_GRADLE_VERSION)"
if [[  -f /usr/illumon/latest/bin/dhconfig  ]]; then
  /usr/illumon/latest/bin/dhconfig schema import --etcd --update --directory /etc/sysconfig/deephaven/plugins/example-plugin/schema/Example | $as_desired_user tee -a ${LOG_FILE} > /dev/null
elif [[ "$dh_version" > "1.20190816" ]]; then
  log_info "Deploying schema /etc/sysconfig/deephaven/plugins/example-plugin/schema/Example"
  $iris deploy_schema --etcd --update --directory /etc/sysconfig/deephaven/plugins/example-plugin/schema/Example | $as_desired_user tee -a ${LOG_FILE} > /dev/null
  log_info "(see ${LOG_FILE} for schema deployment details)"
else
  $iris deploy_schema | $as_desired_user tee -a ${LOG_FILE} > /dev/null
fi

# Record all the schemas we installed so a later deactivation can remove them all.

if ! $as_desired_user test -f /etc/sysconfig/deephaven/plugins/example-plugin/activated/installedSchemas || [[ $($as_irisadmin grep -c "/etc/sysconfig/deephaven/schema/Example" "/etc/sysconfig/deephaven/plugins/example-plugin/activated/installedSchemas") -eq 0 ]]; then
    echo "/etc/sysconfig/deephaven/schema/Example" | $as_desired_user tee -a "/etc/sysconfig/deephaven/plugins/example-plugin/activated/installedSchemas" > /dev/null
fi


log_info ; log_info
log_info "Any new tables / schema changes will not be visible until you run \`monit restart db_dis\` when the system is not performing any data import operations."
log_info ; log_info


finish
