#!/bin/bash

# Copyright (c) 2016-2023 DEEPHAVEN DATA LABS - PATENT PENDING

# This script is automatically generated by iris-plugin-tools.
# Your edits to it will be erased upon upgrade / uninstallation of this plugin.
# If you want to make persistent changes, please contact the developer
# of this plugin, the developer of the plugin tools, or your Deephaven representative.
# *Your changes will be overwritten BEFORE any upgrade operation is performed*.

set -o errexit
set -o pipefail
umask 002


# disable trace logging until after we've redirected stdout and stderr
SCRIPT_DIR="$( cd "$( dirname "${BASH_SOURCE[0]}" )" >/dev/null 2>&1 && pwd )"
SCRIPT_NAME=$(basename "${BASH_SOURCE[0]}")
INSTALL_TIME=${INSTALL_TIME:-$(date "+%Y%m%d%H%M%S")}
export INSTALL_TIME

current_user_id=$(id -u)
current_group_id=$(id -g)
irisadmin_user_id=$(id -u irisadmin)
root_user_id=$(id -u root)
irisadmin_group_id=$(getent group irisadmin | awk -F: '{ print $3 }')
desired_user_id=$(id -u irisadmin)
desired_group_id=$(getent group dbmergegrp | awk -F: '{ print $3 }')

if (( current_user_id == irisadmin_user_id)); then
    as_irisadmin=" "
else
    as_irisadmin="sudo -u irisadmin"
fi

if (( current_user_id == desired_user_id )); then
    if (( current_group_id == desired_group_id )); then
        as_desired_user=" "
    else
        echo "$(id -un):$(id -ug) Cannot run ${SCRIPT_NAME} as irisadmin:dbmergegrp"
        exit 77
    fi
else
    as_desired_user="sudo -u irisadmin  "
fi

if (( current_user_id == root_user_id )); then
    as_admin_user=" "
else
    as_admin_user="sudo "
fi

LOG_DIR="/var/log/deephaven/plugins/example-plugin"
LOG_DIR="${LOG_DIR}/installation"
EXECUTION_DIR="${LOG_DIR}/executions"
LOG_FILE="${LOG_DIR}/${SCRIPT_NAME}-${INSTALL_TIME}.log"
EXECUTION_FILE="${EXECUTION_DIR}/${SCRIPT_NAME}.${INSTALL_TIME}"

# Create LOG_ROOT as irisadmin, then create individual logging directories as the desired user.
$as_irisadmin mkdir -p "${LOG_DIR}"
$as_irisadmin chmod 777 "${LOG_DIR}"
$as_desired_user mkdir -p "${LOG_DIR}"
$as_desired_user mkdir -p "${EXECUTION_DIR}"
$as_irisadmin chmod 755 "${LOG_DIR}"

$as_desired_user touch "${EXECUTION_FILE}"
$as_desired_user touch "${LOG_FILE}"
$as_desired_user chmod 777 "${LOG_FILE}"
$as_desired_user ln -sf "${EXECUTION_FILE}" ${EXECUTION_DIR}/current.${SCRIPT_NAME}.log
$as_desired_user ln -sf "${LOG_FILE}" ${LOG_DIR}/current.${SCRIPT_NAME}.log

$as_desired_user chmod -R 755 "${LOG_DIR}"
$as_desired_user chmod -R 755 "${EXECUTION_DIR}"

### This "exec > >(tee..." will send all script to log file as well as screen, preserving original stdout/err as &3 n &4
exec 3>&1 4>&2 \
     1> >( $as_desired_user tee -a "${LOG_FILE}" >&3  )  \
     2> >( grep --line-buffered -v "+ set +o xtrace" | grep --line-buffered -v "+ log_" | $as_desired_user tee -a "${LOG_FILE}" | grep -E -v "^\+\+* " >&4  )
# The first grep removes + set +o xtrace, the second removes log_* calls; then sends to log file, then erase lines beginning w/ +, then send to "real stderr"




set -o xtrace

function start() {

    log_info "Starting ${SCRIPT_NAME}"
    
    ### Save command
    echo "${SCRIPT_DIR}/${SCRIPT_NAME} ${@}" | $as_desired_user tee "${EXECUTION_FILE}" > /dev/null
    log_debug ""
    log_debug "Script Invoked:"
    log_debug "${SCRIPT_DIR}/${SCRIPT_NAME} ${@}"
    log_debug 

}

function finish() {
    local target="${1:-0}"
    if [ "0" == "$target" ]; then
        target="${SUCCESS_FILE}"
    else
        target="${FAIL_FILE}"
    fi
    log_info "Finishing ${SCRIPT_NAME}"
}

function log_debug() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S") | --- $@" | $as_desired_user tee -a ${LOG_FILE} > /dev/null
    set -o xtrace
}

function log_info() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S")  | --- $@" >&1
    set -o xtrace
}

function log_error() {
    set +o xtrace
    echo "${SCRIPT_NAME} $(date "+%Y-%m-%d %H:%M:%S")  | --- $@" >&2
    set -o xtrace
}

start




# These semantics look a little weird, but this is what rpm does
# (see http://meinit.nl/rpm-spec-prepostpreunpostun-argument-values )

binpath=`dirname $0`

$binpath/afterInstall.sh 2
$binpath/beforeUninstall.sh 1



finish
